//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DSensor
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DSensor.m,v 1.7 2001/01/05 20:50:54 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DSensor.h"
#import "G3DDefines.h"
#import "G3DArchiver.h"
#import "G3DUnarchiver.h"

@implementation G3DSensor

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DSensor *)sensorWithTrigger:(id<G3DTrigger>)aTrigger observer:(id)obs selector:(SEL)aSelector
{
  return [[[G3DSensor alloc] initWithTrigger:aTrigger observer:obs selector:aSelector] autorelease];
}

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithTrigger:(id<G3DTrigger>)aTrigger observer:(id)obs selector:(SEL)aSelector
{
  NSAssert(aTrigger,@"A valid trigger must be specified!");
  NSAssert(obs,@"A valid observer must be specified!");

  if ((self = [super init])) {
    state = YES;

    trigger = [aTrigger retain];
    observer = [obs retain];
    selector = aSelector;
  }
  return self;
}

- (void)dealloc
{
  [observer release];
  [trigger release];

  [super dealloc];
}

//-----------------------------------------------------------------------------
// Triggering
//-----------------------------------------------------------------------------

- (void)triggerIfNeeded:(G3DFrustum *)fr
{
  if ([trigger triggerShouldActivate]) {
    [observer performSelector:selector];
  }
}

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setState:(BOOL)onoff
{
  state = onoff;
}

- (BOOL)state
{
  return state;
}

- (id)observer
{
  return observer;
}

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder
{
  [super encodeWithCoder:aCoder];

  [aCoder encodeObject:trigger];
  [aCoder encodeObject:observer];
  [aCoder encodeValueOfObjCType:@encode(SEL) at:&selector];
  [aCoder encodeValueOfObjCType:@encode(BOOL) at:&state];
}

- (id)initWithCoder:(NSCoder *)aCoder
{
  self = [super initWithCoder:aCoder];

  trigger = [[aCoder decodeObject] retain];
  observer = [[aCoder decodeObject] retain];
  [aCoder decodeValueOfObjCType:@encode(SEL) at:&selector];
  [aCoder decodeValueOfObjCType:@encode(BOOL) at:&state];

  return self;
}

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone
{
  G3DSensor *copy = [super copyWithZone:zone];

  [copy setState:state];
  copy->selector = selector;
  copy->observer = [observer retain];
  copy->trigger = [trigger retain];
  
  return copy;
}

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];

  [archiver encodeObject:trigger named:@"trigger"];
  [archiver encodeObject:observer named:@"observer"];
  [archiver encodeSelector:selector named:@"selector"];

  [archiver encodeBOOL:state named:@"state"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super initWithSceneUnarchiver:unarchiver];
  
  trigger = [[unarchiver decodeObjectNamed:@"trigger"] retain];
  observer = [[unarchiver decodeObjectNamed:@"observer"] retain];
  selector = [unarchiver decodeSelectorNamed:@"selector"];

  [unarchiver decodeBOOL:&state named:@"state"];

  return self;
}

@end






