/* 
   ASEViewr.m

   Simple rendering demo showing ASE import

   Copyright (C) 2000 Philippe C.D. Robert

   Author:  Philippe C.D. Robert <phr@i3dkit.org>
   Date: March 2000
   
   This file is part of the GNU 3DKit

   This file is rleased under the GNU GPL!
*/ 

#import <Foundation/Foundation.h>
#import <RenderKit/RenderKit.h>
#import <SDLKit/G3DSDLKit.h>

#import "G3DASELoader.h"

#define STEP 10
#define ANGLE 3.0f

NSString *aseModel = nil;
BOOL useLighting= YES;

@interface AppController : NSObject
{
  G3DSDLCamera *cam;
  G3DWorld *world;
}

- (void)applicationDidFinishLaunching;
- (void)dealloc;

@end

@implementation AppController

- (void)applicationDidFinishLaunching
{
  NSAutoreleasePool *pool;
  static BOOL sdb = NO;
  G3DLight *lightA;
  G3DGroup *tg;
  G3DTexture *tex;
  G3DVector3f *vec;
  G3DTransform *t;
  G3DGraphNode *imp;
  SDL_Event event;
  G3DSDLEventHandler *dHdl;

  float pos[3] = {0,4,150};
  float to[3] = {0,4,0};
  float up[3] = {0,1,0};

  world = [[G3DWorld alloc] init];

  cam = [[G3DSDLCamera alloc] initWithSize:NSMakeSize(640,480)
			      depth:16 
			      fullscreen:NO];
  [cam setRenderDelegate:self];
  [cam setWorld:world];
  [cam setUseTwoSideModel:YES];
  [cam setEnableLighting:YES];
  [[cam frustum] setNear:1.0f far:1000.0f];

  // Do some framerate measuring
  [G3DSDLCamera measureFramerates:YES];

  // Finally initialise the camera and feed GL
  [cam setupCamera];

  // =====================================
  // The lighting
  // =====================================

  pool = [[NSAutoreleasePool alloc] init];

  if (useLighting) {
    NSLog(@"Initialising 2 global lights...");

    vec = [G3DVector3f vector3fWithX:1.0f y:0.0f z:0.0f];
    lightA = [G3DLight distantLightWithDirection:vec];
    [lightA setDiffuse:[G3DColour colourWithR:0.8 g:0.3 b:0.2 a:0.8]];
    [lightA setAmbient:[G3DColour colourWithR:0.8 g:0.4 b:0.5 a:0.8]];
    [world addDynamicObject:lightA];
    
    vec = [G3DVector3f vector3fWithX:0.3f y:0.4f z:0.0f];
    lightA = [G3DLight distantLightWithDirection:vec];
    //[lightA setDiffuse:[G3DColour colourWithR:0.4 g:0.5 b:0.8 a:0.8]];
    //[lightA setAmbient:[G3DColour colourWithR:0.2 g:0.3 b:0.8 a:0.8]];
    //[lightA setSpecular:[G3DColour colourWithR:0.0 g:0.0 b:0.0 a:1.0]];
    [world addDynamicObject:lightA];
  }
  else {
    [cam setEnableLighting:NO];

    NSLog(@"disable lighting!");
  }

  // =====================================
  // The scenegraph structure
  // =====================================

  NSLog(@"Setting up the scenegraph...");

  tg = [[G3DGroup alloc] init];

  if ([[NSFileManager defaultManager] isReadableFileAtPath:aseModel]) {
    imp = [[G3DASELoader defaultLoader] objectTreeWithContentsOfFile:aseModel];
    if (imp) {
      vec = [G3DVector3f vector3fWithX:-1.0f y:0.0f z:0.0f];
      t = [[G3DTransform alloc] initWithScale:nil hpr:vec translation:nil];
      [imp setTransformation:t];
      [imp setName:aseModel];
      [t release];
      
      if ([imp isKindOfClass:[G3DShape class]]) {
	if (![[(G3DShape *)imp state] isFeatureEnabled:G3D_Lighting]) {
	  [[(G3DShape *)imp state] enableFeature:G3D_Lighting];
	}
      }
      
      [tg addChild:imp];
      
      NSLog(@"<%@ %x>: import of %@ OK!",[self class],self,[imp description]);
    }
    else {
      [NSException raise:@"SDLDemoException" 
		   format:@"Could not import %@",aseModel];
    }
  }
  else {
    NSLog(@"%@ does not exist at the specified path!",aseModel);
  }

  /*
  {
    int k;

    NSArray *a = [tg leaves];

    for (k=0;k<[a count];k++) {
      G3DShape *g = [a objectAtIndex:k];

      [g setEnableLighting:YES];      
      NSLog(@"Object named %@",[g name]);
    }
  }
  */

  [world addStaticObject:tg];
  [tg release];

  // =====================================
  // Fog
  // =====================================

  [cam setCamera:pos toward:to up:up];

  [cam  setFog:[G3DFog fog]];
  [[cam fog] setStart:50.0];
  [[cam fog] setEnd:1000.0];

  // =====================================
  // Main Loop
  // =====================================

  [cam preprocessScene];
  [pool release];

  NSLog(@"Starting the rendering...");

  dHdl = [G3DSDLEventHandler defaultHandler];
  [dHdl setKeyRepeatWithDelay:1 interval:1];

  // Ignore the key up events
  SDL_EventState(SDL_KEYUP, SDL_IGNORE);

  while (![dHdl isKeyPressed:SDLK_ESCAPE]) {
    pool = [[NSAutoreleasePool alloc] init];

    if ([dHdl getEvent:&event]) {
      switch (((event.key).keysym).sym) {
      case SDLK_UP:
	[cam moveX:0.0f y:0.0f z:-STEP];
	break;
      case SDLK_LEFT:
	[cam moveX:-STEP y:0.0f z:0.0f];
	break;
      case SDLK_RIGHT:
	[cam moveX:STEP y:0.0f z:0.0f];
	break;
      case SDLK_DOWN:
	[cam moveX:0.0f y:0.0f z:STEP];
	break;
      case SDLK_a:
	[cam moveX:0.0f y:STEP z:0.0f];
	break;
      case SDLK_z:
	[cam moveX:0.0f y:-STEP z:0.0f];
	break;
      case SDLK_DELETE:
	[cam rotateY:-ANGLE];
	break;
      case SDLK_PAGEDOWN:
	[cam rotateY:ANGLE];
	break;
      case SDLK_HOME:
	[cam rotateX:ANGLE];
	break;
      case SDLK_END:
	[cam rotateX:-ANGLE];
	break;
      case SDLK_INSERT:
	[cam rotateZ:ANGLE];
	break;
      case SDLK_PAGEUP:
	[cam rotateZ:-ANGLE];
	break;
      case SDLK_c:
	[cam setCullingEnabled:([cam cullingEnabled])?NO:YES];
	break;
      case SDLK_m:
	[[G3DRenderEngine renderEngine] toggleRenderMode];
	break;
      default:
	break;
      }
    }
    
    ((event.key).keysym).sym = SDLK_LAST;
    [cam render];

    [pool release];
  }
  
  [[G3DRenderEngine renderEngine] cleanup];
  exit(0);
}

- (void)dealloc
{
  [cam release];
  [world release];

  [super dealloc];
}

@end

int main(int argc, char **argv)
{
  AppController *controller;
  id pool = [NSAutoreleasePool new];

  if (argc < 3) {
    NSLog(@"Usage: opentool SDLDemo <--model ModelName> [--lighting YES|NO]");
    [pool release];
    return 0;
  }
  
  aseModel = [[NSString stringWithCString:argv[2]] retain];

  if (argc == 5) {
    NSString *arg = [NSString stringWithCString:argv[4]];

    useLighting = ([arg isEqualToString:@"YES"]) ? YES : NO;
  }

  NSLog(@"Rendering model %@",aseModel);

  // The delegate
  controller = [[AppController alloc] init];
  [controller applicationDidFinishLaunching];

  // The end...
  [controller release];
  [pool release];

  return 0;
}




