//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DASEObject
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DASEObject.h,v 1.1 2001/07/15 14:03:37 probert Exp $
//-----------------------------------------------------------------------------

/*
 * No vertex nor face normals are imported yet...
 *
 */

#ifndef __G3DASEObject_h_INCLUDE
#define __G3DASEObject_h_INCLUDE

#import <Foundation/Foundation.h>

typedef struct _G3D_Face {
  unsigned index;
  unsigned a;
  unsigned b;
  unsigned c;
  unsigned ab;
  unsigned bc;
  unsigned ca;
} G3D_Face;

@interface G3DASEObject : NSObject
{
  NSString *name;

  float tm[16];

  float rotationAxis[3];
  float rotationAngle;

  int timeValue;
  unsigned vertexCount;
  unsigned normalCount;
  unsigned faceCount;

  float *vertices; // array of 4 floats
  float *vNormals; // array of 4 floats
  G3D_Face *faces;  // array of structs

  int materialRef;

  BOOL hasVertexNormals;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DASEObject *)aseObjectWithFaces:(unsigned)fc vertices:(unsigned)vc;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithFaces:(unsigned)fc vertices:(unsigned)vc;
- (void)dealloc;

//-----------------------------------------------------------------------------
// Accessors
//-----------------------------------------------------------------------------

- (void)setName:(NSString *)aName;
- (NSString *)name;

- (unsigned)vertexCount;
- (unsigned)normalCount;
- (unsigned)faceCount;

- (void)setHasVertexNormals:(BOOL)yn;
- (BOOL)hasVertexNormals;

- (void)setTransformationWithElements:(float *)values;
- (void)getTransformationWithElements:(float *)values;

- (float *)vertices;
- (float *)vNormals;
- (G3D_Face *)faces;

- (void)setVertex:(float *)vtx atIndex:(unsigned)ind;
- (void)getVertex:(float *)vtx atIndex:(unsigned)ind;

- (float *)vertexAtIndex:(unsigned)ind;
- (float *)normalAtIndex:(unsigned)ind;

- (void)setFace:(G3D_Face *)face atIndex:(unsigned)ind;
- (void)getFace:(G3D_Face *)face atIndex:(unsigned)ind;

- (void)setMaterialRef:(int)ref;
- (int)materialRef;

@end

extern NSString *G3DASEException;

#endif




