//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DASEObject
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DASEObject.m,v 1.1 2001/07/15 14:03:37 probert Exp $
//-----------------------------------------------------------------------------

#import "G3DASEObject.h"

#import <GeometryKit/G3DVectorFunc.h>
#import <GeometryKit/G3DMatrixFunc.h>

NSString *G3DASEException = @"G3DASEException";

@implementation G3DASEObject

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DASEObject *)aseObjectWithFaces:(unsigned)fc vertices:(unsigned)vc
{
  return [[[G3DASEObject alloc] initWithFaces:fc vertices:vc] autorelease];
}

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithFaces:(unsigned)fc vertices:(unsigned)vc
{
  NSAssert(vc>2,@"At least 3 vertices are required!");
  NSAssert(fc>0,@"At least 1 face is required!");

  if ((self = [super init])) {
    vertexCount = vc;
    vertices = malloc(vc * 4 * sizeof(float));

    if (vertices == NULL) {
      [NSException raise:G3DASEException format:@"malloc() failed?!"];
      [self autorelease];
      return nil;
    }

    normalCount = vc;
    vNormals = malloc(vc * 3 * sizeof(float));
    if (vNormals == NULL) {
      [NSException raise:G3DASEException format:@"malloc() failed?!"];
      [self autorelease];
      return nil;
    }
    hasVertexNormals = NO;

    faceCount = fc;
    faces = malloc(fc * sizeof(G3D_Face));
    if (faces == NULL) {
      [NSException raise:G3DASEException format:@"malloc() failed?!"];
      [self autorelease];
      return nil;
    }

    materialRef = -1;
  }
  return self;
}

- (void)dealloc
{
  free(vertices);
  free(vNormals);
  free(faces);

  [name release];

  [super dealloc];
}

//-----------------------------------------------------------------------------
// Accessors
//-----------------------------------------------------------------------------

- (void)setName:(NSString *)aName
{
  [name autorelease];
  name = [aName copy];
}

- (NSString *)name
{
  return name;
}

- (unsigned)vertexCount
{
  return vertexCount;
}

- (unsigned)normalCount
{
  return normalCount;
}

- (unsigned)faceCount
{
  return faceCount;
}

- (void)setHasVertexNormals:(BOOL)yn
{
  hasVertexNormals = yn;
}

- (BOOL)hasVertexNormals
{
  return hasVertexNormals;
}

- (void)setTransformationWithElements:(float *)values
{
  G3DCopyMatrix4fv(tm,values);
}

- (void)getTransformationWithElements:(float *)values
{
  G3DCopyMatrix4fv(values,tm);
}

- (float *)vertices
{
  return vertices;
}

- (float *)vNormals
{
  return vNormals;
}

- (G3D_Face *)faces
{
  return faces;
}

- (void)setVertex:(float *)vtx atIndex:(unsigned)ind
{
}

- (void)getVertex:(float *)vtx atIndex:(unsigned)ind
{
  G3DCopyVector4fv(vtx,&vertices[4*ind]);
}

- (float *)vertexAtIndex:(unsigned)ind
{
  //DLOG(@"Vertex at %d is %f %f %f %f",ind,vertices[4*ind],vertices[4*ind+1],vertices[4*ind+2],vertices[4*ind+3]);

  return &vertices[4*ind];
}

- (float *)normalAtIndex:(unsigned)ind
{
  //DLOG(@"Normal at %d is %f %f %f %f",ind,vNormals[4*ind],vNormals[4*ind+1],vNormals[4*ind+2],vNormals[4*ind+3]);

  return &vNormals[3*ind];
}

- (void)setFace:(G3D_Face *)face atIndex:(unsigned)ind
{
}

- (void)getFace:(G3D_Face *)face atIndex:(unsigned)ind
{
}

- (void)setMaterialRef:(int)ref
{
  materialRef = ref;
}

- (int)materialRef
{
  return materialRef;
}

@end






