//-----------------------------------------------------------------------------
// Project            	RenderKit
// Protocol		G3DVtxArray+ASEExtensions
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Fri Aug  4 13:22:05 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DVtxArray+ASEExtensions.m,v 1.1 2001/07/15 14:03:37 probert Exp $
//-----------------------------------------------------------------------------

#import "G3DVtxArray+ASEExtensions.h"
#import "G3DASEObject.h"

#import <RenderKit/RenderKit.h>

NSString *G3DVtxArrayASEImportException = @"G3DVtxArrayASEImportException";

@implementation G3DVtxArray (ASEExtensions)

+ (G3DVtxArray *)arrayWithASEObject:(G3DASEObject *)data
{
  return [[[G3DVtxArray alloc] initWithASEObject:data] autorelease];
}

- (id)initWithASEObject:(G3DASEObject *)data
{
  NSAssert(data,@"No valid ASE object!");

  if ((self = [self initWithMode:G3D_TRIANGLES count:[data faceCount]*3])) {
    int i;
    int _index = 0;
    int iA, iB, iC;
    G3D_Face *faces = [data faces];

    /*
     * Please, optimise this ASAP...;-)
     */

    if (![data hasVertexNormals]) {
      for (i=0;i<[data faceCount];i++,_index+=3) {
	iA = faces[i].a;
	iB = faces[i].b;
	iC = faces[i].c;
	
	[self setCoordinate:[data vertexAtIndex:iA] atIndex:(_index)];
	[self setCoordinate:[data vertexAtIndex:iB] atIndex:(_index+1)];
	[self setCoordinate:[data vertexAtIndex:iC] atIndex:(_index+2)];
      }
      [self setUseVertexNormals:NO];
    }
    else {
      for (i=0;i<[data faceCount];i++,_index+=3) {
	iA = faces[i].a;
	iB = faces[i].b;
	iC = faces[i].c;
	
	[self setCoordinate:[data vertexAtIndex:iA] atIndex:(_index)];
	[self setCoordinate:[data vertexAtIndex:iB] atIndex:(_index+1)];
	[self setCoordinate:[data vertexAtIndex:iC] atIndex:(_index+2)];
	
	// Assuming they are normalised!
	[self setNormal:[data normalAtIndex:iA] atIndex:(_index)];
	[self setNormal:[data normalAtIndex:iB] atIndex:(_index+1)];
	[self setNormal:[data normalAtIndex:iC] atIndex:(_index+2)];
      }
      [self setUseVertexNormals:YES];
    }
  }
  return self;
}

@end




