/* 
   G3DWaterSurface.m

   Simple water animation

   Copyright (C) 2000 Philippe C.D. Robert

   Author:  Philippe C.D. Robert <robert@iam.unibe.ch>
   Date: March 2000
   
   This file is part of the interactive 3DKit

   This file is rleased under the GNU GPL!
*/ 

#ifndef __G3DWaterSurface_h_INCLUDE
#define __G3DWaterSurface_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DGeometryBackend.h>

@class G3DShape;

@interface G3DWaterSurface : NSObject <G3DGeometryBackend,NSCoding,NSCopying>
{
  int _sl;

  float *water1;    // current, velocity, after integration
  float *water2;
  float *water3;
  float *stage[3];

  float lowering;   // To deaden the waves
  float altitude;   // Factor that is added to an altered vertex
  int dropRate;     // Every n frame a drop is dropped...
  int updateRate;   // Every n frame the water gets calculated...

  float ang_x;
  float ang_y;
  float inc_ang_x;
  float inc_ang_y;
  float scale;

  int current;         // Current stage of water

  float ang_circle;    // The angle of the animated focus
  float focusVelocity; // Velocity of the focus (angular)

  BOOL hasWaves;       // Default is NO

  G3DShape *shape;

  @private
    
  float _f;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DWaterSurface *)surfaceWithSlices:(int)sl scale:(float)sc;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithSlices:(int)sl scale:(float)sc;
- (void)dealloc;

//-----------------------------------------------------------------------------
// Water rendering
//-----------------------------------------------------------------------------

- (void)updateWater;
- (void)animate;

- (void)setHasWaves:(BOOL)yn;
- (BOOL)hasWaves;

- (void)setDropRate:(int)val;
- (int)dropRate;

- (void)setLowering:(float)val;
- (float)lowering;

- (void)setAltitude:(float)val;
- (float)altitude;

//-----------------------------------------------------------------------------
// Geometry backend
//-----------------------------------------------------------------------------

- (void)drawGeometry;

- (void)precompile;

- (int)vertexCount;
- (void)getMin:(float *)min max:(float *)max;
- (void)setShape:(G3DShape *)aShape;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DWaterSurfaceException;

#endif







